@@ -89,6 +89,7 @@ class InstallerWindow:
         # load the window object
         self.window = self.builder.get_object("main_window")
         self.window.connect("delete-event", self.quit_cb)
+        self.scale = self.window.get_scale_factor()
 
         # wizard pages
         (self.PAGE_LANGUAGE,
@@ -108,7 +109,10 @@ class InstallerWindow:
         self.builder.get_object("button_back").connect("clicked", self.wizard_cb, True)
         self.builder.get_object("button_quit").connect("clicked", self.quit_cb)
 
-        col = Gtk.TreeViewColumn("", Gtk.CellRendererPixbuf(), pixbuf=2)
+        ren = Gtk.CellRendererPixbuf()
+        col = Gtk.TreeViewColumn("Flags", ren)
+        col.add_attribute(ren, "pixbuf", 2)
+        col.set_cell_data_func(ren, self.data_func_surface)
         self.builder.get_object("treeview_language_list").append_column(col)
         ren = Gtk.CellRendererText()
         self.language_column = Gtk.TreeViewColumn(_("Language"), ren, text=0)
@@ -380,7 +384,7 @@ class InstallerWindow:
         # Refresh the current title and help question in the page header
         self.activate_page(self.PAGE_LANGUAGE)
 
-    def validate_entry(self, entry, validate_regex, len_range, check_lower=False, check_match=None, also_true=True):
+    def validate_entry(self, entry, validate_regex, len_range, check_lower=False, check_space=False, check_hyphens=False, check_match=None, also_true=True):
         error = None
         value = entry.props.text
 
@@ -391,12 +395,15 @@ class InstallerWindow:
             if len(value) == 0:
                 error = _("This field cannot be empty.")
                 break
-            if re.search(WHITESPACE_REGEX, value):
+            if check_space and re.search(WHITESPACE_REGEX, value):
                 error = _("This field may not contain space characters.")
                 break
             if check_lower and re.search(HAS_LOWER_REGEX, value):
                 error = _("This field must be lower case.")
                 break
+            if check_hyphens and value[0] in ("-", ".") or value[-1] in ("-", "."):
+                error = _("This field cannot start or end with a hyphen or period.")
+                break
             if len_range:
                 if len(value) < len_range[0]:
                     error = _("The entry is too short.")
@@ -455,14 +462,14 @@ class InstallerWindow:
         except:
             pass
 
-        self.validate_entry(self.builder.get_object("entry_username"), USERNAME_REGEX, USERNAME_LENGTH, check_lower=True)
+        self.validate_entry(self.builder.get_object("entry_username"), USERNAME_REGEX, USERNAME_LENGTH, check_lower=True, check_space=True)
         self.validate_user_page()
         self.setup.print_setup()
 
     def assign_username(self, entry, prop):
         self.setup.username = entry.props.text
 
-        self.validate_entry(entry, USERNAME_REGEX, USERNAME_LENGTH, check_lower=True)
+        self.validate_entry(entry, USERNAME_REGEX, USERNAME_LENGTH, check_lower=True, check_space=True)
         self.validate_user_page()
         self.setup.print_setup()
 
@@ -473,9 +480,9 @@ class InstallerWindow:
         # and there are length rules for each segment and the total length. Do the check here, then pass along
         # such length requirements as to trigger too short/long warnings.
         if self.hostname_too_long(self.setup.hostname):
-            self.validate_entry(entry, HOSTNAME_REGEX, (0, 0), check_lower=True)
+            self.validate_entry(entry, HOSTNAME_REGEX, (0, 0), check_space=True, check_hyphens=True)
         else:
-            self.validate_entry(entry, HOSTNAME_REGEX, (HOSTNAME_MIN_LENGTH, 9999), check_lower=True)
+            self.validate_entry(entry, HOSTNAME_REGEX, (HOSTNAME_MIN_LENGTH, 9999), check_space=True, check_hyphens=True)
 
         self.validate_user_page()
         self.setup.print_setup()
@@ -574,13 +581,18 @@ class InstallerWindow:
     def show_customwarning(self, widget):
         self.activate_page(self.PAGE_CUSTOMWARNING)
 
+    def data_func_surface(self, column, cell, model, iter_, *args):
+        pixbuf = model.get_value(iter_, 2)
+        surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, self.scale)
+        cell.set_property("surface", surface)
+
     def flag(self, lang, language, ccode):
         print (language.lower())
         for filename in [language.lower(), ccode.lower(), "united_nations"]:
             path = f"/usr/share/circle-flags-svg/{filename}.svg"
             if os.path.exists(path):
                 break
-        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(path, 18, 18)
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(path, -1, 18 * self.scale)
         return pixbuf
 
     def build_lang_list(self):
